﻿using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Authentications;
using XF.Domain.Repository;
using Hotline.Application.Caselibrary;
using Hotline.Share.Dtos.Caselibrary;
using SqlSugar;
using Hotline.CaseLibrary;
using XF.Domain.Exceptions;
using Hotline.Share.Dtos;
using Hotline.Share.Tools;
using Hotline.Share.Enums.Caselibrary;
using Hotline.Application.ExportWord;
using Hotline.Application.Tools;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 案例库
    /// </summary>
    public class CaseController : BaseController
    {

        #region 注入

        private readonly IMapper _mapper;
        private readonly ISessionContext _sessionContext;
        private readonly ICaseApplication _CaseApplication;
        private readonly IRepository<CaseType> _CaseTypeRepository;
        private readonly IRepository<CaseList> _CaseListRepository;
        private readonly IRepository<CaseCollect> _CaseCollectRepository;
        private readonly IWordHelperService _wordHelperService;


        public CaseController(
           IMapper mapper,
           ISessionContext sessionContext,
           ICaseApplication CaseApplication,
           IRepository<CaseType> CaseTypeRepository,
           IRepository<CaseList> CaseListRepository,
           IRepository<CaseCollect> CaseCollectRepository,
           IWordHelperService wordHelperService)
        {
            _mapper = mapper;
            _sessionContext = sessionContext;
            _CaseApplication = CaseApplication;
            _CaseTypeRepository = CaseTypeRepository;
            _CaseListRepository = CaseListRepository;
            _CaseCollectRepository = CaseCollectRepository;
            _wordHelperService = wordHelperService;
        }

        #endregion

        #region 案例库类型管理

        /// <summary>
        /// 获取列表层级分类
        /// </summary>
        /// <param name="IsEnable">是否启用</param>
        /// <returns></returns>
        [HttpGet("type/treelist")]
        public async Task<List<CaseTypeDto>> QueryAllTreeList(bool? IsEnable)
        {
            return await _CaseTypeRepository.Queryable()
                .WhereIF(IsEnable.HasValue, x => x.IsEnable == IsEnable)
                .Where(x => SqlFunc.Subqueryable<CaseTypeOrg>().Where(to => to.TypeId == x.Id).Any() ||
                SqlFunc.Subqueryable<CaseTypeOrg>().Where(to => to.TypeId == x.Id).NotAny()
                )
                .Select(x => new CaseTypeDto()
                {
                    Id = x.Id.SelectAll()
                }
                )
                .OrderBy(x => x.Sort).ToTreeAsync(it => it.children, it => it.ParentId, null, it => it.Id);
        }

        /// <summary>
        /// 新增
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("type/add")]
        public async Task<string> AddType([FromBody] AddCaseTypeDto dto)
        {
            return await _CaseApplication.AddTypeAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 编辑
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("type/update")]
        public async Task UpdateType([FromBody] UpdateCaseTypeDto dto)
        {
            await _CaseApplication.UpdateTypeAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 查询详情
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpGet("type/info/{Id}")]
        public async Task<CaseType> GetType(string Id)
        {
            var types = await _CaseTypeRepository.Queryable()
                .Includes(x => x.CaseTypeOrgs)   // 填充子对象
                .Where(x => x.Id == Id)
                .FirstAsync(HttpContext.RequestAborted);
            if (types is null)
                throw UserFriendlyException.SameMessage("查询失败！");
            return types;
        }

        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="Id"></param>
        /// <returns></returns>
        [HttpDelete("type/remove/{Id}")]
        public async Task RemoveType(string Id)
        {
            await _CaseApplication.RemoveTypeAsync(Id, HttpContext.RequestAborted);
        }

        #endregion

        #region 案例库管理

        /// <summary>
        /// 案例库分类列表
        /// </summary>
        /// <param name="IsEnable"></param>
        /// <returns></returns>
        [HttpGet("list/treelist")]
        public async Task<List<CaseTypeDto>> QueryAllCaseTypeTreeList(bool? IsEnable)
        {
            return await _CaseTypeRepository.Queryable()
                .WhereIF(IsEnable.HasValue, x => x.IsEnable == IsEnable)
                .Where(x => SqlFunc.Subqueryable<CaseTypeOrg>().Where(to => to.TypeId == x.Id).Any() ||
                SqlFunc.Subqueryable<CaseTypeOrg>().Where(to => to.TypeId == x.Id).NotAny()
                )
                .Select(x => new CaseTypeDto()
                {
                    Id = x.Id.SelectAll(),
                    CaseNum = SqlFunc.Subqueryable<CaseRelationType>().LeftJoin<CaseList>((kr, k) => kr.CaseId == k.Id)
                         .Where((kr, k) => kr.CaseTypeSpliceName.StartsWith(x.SpliceName))
                         .DistinctCount(kr => kr.CaseId)
                }
                )
                .OrderBy(x => x.Sort).ToTreeAsync(it => it.children, it => it.ParentId, null, it => it.Id);
        }

        /// <summary>
        /// 案例库列表
        /// </summary>
        /// <param name="pagedDto"></param>
        /// <returns></returns>
        [HttpGet("list")]
        public async Task<PagedDto<CaseDataDto>> QueryAllCaseList([FromQuery] CaseListDto pagedDto)
        {
            return (await _CaseApplication.QueryAllCaseListAsync(pagedDto, HttpContext.RequestAborted)).ToPaged();
        }

        /// <summary>
        /// 案例库草稿
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("list/draft")]
        public async Task<string> CaseDraft([FromBody] AddCaseListDto dto)
        {
            dto.Status = ECaseStatus.NewDrafts;
            return await _CaseApplication.AddCaseAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库草稿修改
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("list/draftupdate")]
        public async Task UpdateCaseDraft([FromBody] UpdateCaseListDto dto)
        {
            dto.Status = ECaseStatus.NewDrafts;
            await _CaseApplication.UpdateCaseAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库草稿上架到审核
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("list/draftadd")]
        public async Task AddCaseDraft([FromBody] UpdateCaseListDto dto)
        {
            dto.ApplyStatus = ECaseApplyStatus.Add;
            dto.Status = ECaseStatus.Auditing;
            await _CaseApplication.UpdateCaseAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库新增到审核
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("list/add")]
        public async Task<string> AddCase([FromBody] AddCaseListDto dto)
        {
            dto.ApplyStatus = ECaseApplyStatus.Add;
            dto.Status = ECaseStatus.Auditing;
            return await _CaseApplication.AddCaseAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库编辑提交到审核
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("list/update")]
        public async Task UpdateCase([FromBody] UpdateCaseListDto dto)
        {
            dto.ApplyStatus = ECaseApplyStatus.Update;
            dto.Status = ECaseStatus.Auditing;
            await _CaseApplication.UpdateCaseAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库下架
        /// </summary>
        /// <param name="Id">案例库ID</param>
        /// <returns></returns>
        [HttpGet("list/offshelf/{Id}")]
        public async Task OffshelfCase(string Id)
        {
            UpdateCaseListDto dto = new UpdateCaseListDto();
            dto.Id = Id;
            dto.ApplyStatus = ECaseApplyStatus.Offshelf;
            dto.Status = ECaseStatus.Auditing;

            await _CaseApplication.AuditCaseAsync(dto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库审核（新增、修改、下架）
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("list/examin")]
        public async Task ExaminCase([FromBody] AuditCaseListDto dto)
        {
            var Case = await _CaseListRepository.GetAsync(dto.Id);
            if (Case == null)
                throw UserFriendlyException.SameMessage("案例库查询失败");

            var CaseDto = _mapper.Map<UpdateCaseListDto>(Case);

            if (dto.State == 0)
            {//不同意
                CaseDto.Status = ECaseStatus.Revert;
            }
            else if (dto.State == 1)
            {//同意 
                if (CaseDto.ApplyStatus == ECaseApplyStatus.Add)
                {
                    CaseDto.Status = ECaseStatus.OnShelf;
                }
                if (CaseDto.ApplyStatus == ECaseApplyStatus.Update)
                {
                    CaseDto.Status = ECaseStatus.OnShelf;
                    CaseDto.UpdateTime = DateTime.Now;
                }
                if (CaseDto.ApplyStatus == ECaseApplyStatus.Offshelf)
                {
                    CaseDto.Status = ECaseStatus.OffShelf;
                }
            }
            CaseDto.Id = dto.Id;
            CaseDto.ExaminTime = DateTime.Now;
            CaseDto.ExaminManId = _sessionContext.UserId;
            CaseDto.ExaminOrganizeId = _sessionContext.OrgId;

            await _CaseApplication.AuditCaseAsync(CaseDto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库详情
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpGet("list/info")]
        public async Task<CaseInfoDto> GetCase([FromBody] PvCaseListDto dto)
        {
            return await _CaseApplication.GetCaseAsync(dto.Id, dto.IsAddPv, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 案例库评分
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("list/score")]
        public async Task ScoreCase([FromBody] PvCaseListDto dto)
        {
            var collect = await _CaseCollectRepository.GetAsync(x => x.CaseId == dto.Id && x.CreatorId == _sessionContext.UserId);
            if (collect != null)
            {
                if (collect.Score > 0)
                    throw UserFriendlyException.SameMessage("当前知识已经评分");

                collect.Score = dto.Score;
                await _CaseCollectRepository.UpdateAsync(collect, HttpContext.RequestAborted);
            }
            else
            {
                collect = new CaseCollect();
                collect.CaseId = dto.Id;
                collect.Score = dto.Score;
                await _CaseCollectRepository.AddAsync(collect, HttpContext.RequestAborted);
            }

            //计算总分
            var sugar = _CaseCollectRepository.Queryable().Where(x => x.CaseId == dto.Id);
            var count = await sugar.CountAsync();
            var collects = await sugar.SumAsync(x => x.Score);
            var scoreTemp = collects / count;
            var Case = await _CaseListRepository.GetAsync(x => x.Id == dto.Id);
            if (Case != null)
            {
                Case.Score = decimal.Round(scoreTemp.Value, 1);
                await _CaseListRepository.UpdateAsync(Case, HttpContext.RequestAborted);
            }
        }

        /// <summary>
        /// 案例库查重
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("list/exist")]
        public async Task<bool> ExistCase([FromBody] CaseExistDto dto)
        {
            var any = await _CaseListRepository.Queryable()
                .Where(x => x.Status == ECaseStatus.Auditing || x.Status >= ECaseStatus.OnShelf)
                .WhereIF(!string.IsNullOrEmpty(dto.Title), x => x.Title.Equals(dto.Title))
                .WhereIF(!string.IsNullOrEmpty(dto.Abstract), x => x.Abstract.Equals(dto.Abstract))
                .WhereIF(!string.IsNullOrEmpty(dto.Describe), x => x.Describe.Equals(dto.Describe))
                .WhereIF(!string.IsNullOrEmpty(dto.Result), x => x.Result.Equals(dto.Result))
                .WhereIF(!string.IsNullOrEmpty(dto.Reason), x => x.Reason.Equals(dto.Reason))
                .WhereIF(!string.IsNullOrEmpty(dto.Id), x => x.Id != dto.Id)
                .AnyAsync();
            return any;
        }

        /// <summary>
        /// 案例库导出
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("list/info/export")]
        public async Task<IActionResult> CaseInfoExport([FromBody] CaseInfoExportDto dto)
        {
            if (dto.Ids.Length > 1)
            {
                var streams = await _CaseApplication.CaseInfoListExportAsync(dto, HttpContext.RequestAborted);
                byte[] fileBytes = _wordHelperService.ConvertZipStream(streams);
                var name = DateTime.Now.ToString("yyyyMMddHHmmss");
                return File(fileBytes, "application/octet-stream", $"{name}.zip");
            }
            var info = await _CaseListRepository.GetAsync(dto.Ids[0]) ?? throw UserFriendlyException.SameMessage("案例不存在");
            return info.Result.HtmlToStream(dto.FileType).GetFileStreamResult(dto.FileType, info.Title, false);
        }

        #endregion

        #region 案例库检索

        /// <summary>
        /// 案例库列表前10
        /// </summary>
        /// <returns></returns>
        [HttpGet("list/top10")]
        public async Task<List<CasePageViewDto>> QueryTop10CaseList()
        {
            return await _CaseListRepository.Queryable()
                .Take(10)
                .Where(x => x.Status == ECaseStatus.OnShelf)
                .Select(x => new CasePageViewDto
                {
                    Id = x.Id,
                    Title = x.Title,
                    PageView = x.PageView
                })
                .OrderBy(x => x.PageView, OrderByType.Desc)
                .ToListAsync();
        }

        #endregion
    }
}
